@@ -4,46 +4,50 @@
 """
 import operator
 import sys
-from collections.abc import (
-    Iterable,
-    Sized,
-    Callable,
-    Generator,
-)
 from itertools import (
-    chain,
-    tee,
-    repeat,
     accumulate,
+    chain,
     combinations,
     combinations_with_replacement,
+    compress,
     cycle,
     dropwhile,
     filterfalse,
-    zip_longest,
-    islice,
-    compress,
     groupby,
+    islice,
+    repeat,
     starmap,
+    takewhile,
+    tee,
+    zip_longest,
 )
 from typing import (
     Any,
+    Callable,
+    ClassVar,
+    Generator,
     Generic,
-    overload,
-    Optional,
-    Union,
+    Hashable,
+    Iterable,
+    Iterator,
     NoReturn,
-    TypeVar,
+    Optional,
     Reversible,
     Sequence,
+    Sized,
+    SupportsIndex,
+    TypeVar,
+    Union,
+    overload,
 )
 
 __all__ = ["ArkoWrapper"]
 
-M = TypeVar("M")
+Wrapper = TypeVar("Wrapper", bound="ArkoWrapper")
 T = TypeVar("T")
 E = TypeVar("E")
 C = TypeVar("C")
+default_max = sys.maxsize
 
 
 class ArkoWrapper(Generic[T]):
@@ -61,7 +65,7 @@ class ArkoWrapper(Generic[T]):
     # noinspection PyTypeChecker
     def __init__(
             self, iterable: Optional[Union[Iterable[T], T]] = None, *,
-            max_operate_times: Optional[int] = sys.maxsize
+            max_operate_times: Optional[int] = default_max
     ) -> NoReturn:
         if isinstance(iterable, Iterable):
             self.__root__ = iterable
@@ -72,29 +76,31 @@ class ArkoWrapper(Generic[T]):
 
         if max_operate_times <= 0:
             raise ValueError(f"Requires a positive number: {max_operate_times}")
+        elif max_operate_times > default_max:
+            raise ValueError(f"'max_operate_times' cannot exceed {default_max}")
         self._max = max_operate_times
 
     def __str__(self) -> str:
         return str(self.__root__)
 
     def __repr__(self) -> str:
-        return f"<{self.__class__.__name__}:{self.__root__}>"
+        return f"<{self.__class__.__name__} " + "{" + f"{self.__root__}" + "}>"
 
     def _tee(self) -> Iterable[T]:
         """将已有迭代器分裂一次"""
         result, self.__root__ = tee(self.__root__)
         return result
 
-    def _max_gen(self) -> Generator[T]:
+    def _max_gen(self) -> Iterator:
         """将自己的迭代器现在某个范围内"""
         iter_values = iter(self._tee())
         try:
             for _ in range(self._max):
                 yield next(iter_values)
         except StopIteration:
-            ...
+            pass
 
-    def __add__(self: M, other: E) -> "M[Union[T, E]]":
+    def __add__(self: Wrapper, other: Union[E, Iterable[E]]) -> Wrapper:
         """实现加法操作。返回两个实列组成的新的迭代器的ArkoWrapper
 
         Args:
@@ -104,24 +110,20 @@ class ArkoWrapper(Generic[T]):
             ArkoWrapper: 返回两个实列组成的新的迭代器的ArkoWrapper
         """
 
-        def generate() -> Generator:
+        def generate() -> Iterator[Union[T, E]]:
             yield from self._tee()
-            if isinstance(other, ArkoWrapper):
-                yield from other._tee()
-            elif isinstance(other, Iterable):
+            if isinstance(other, Iterable):
                 yield from tee(other)[0]
             else:
                 yield other
 
         return self.__class__(generate())
 
-    def __radd__(self: M, other: E) -> "M[Union[T, E]]":
+    def __radd__(self: Wrapper, other: Union[Iterable[E], E]) -> Wrapper:
         """实现反射加法操作。"""
 
-        def generate() -> Generator:
-            if isinstance(other, ArkoWrapper):
-                yield from other._tee()
-            elif isinstance(other, Iterable):
+        def generate() -> Iterator[Union[T, E]]:
+            if isinstance(other, Iterable):
                 yield from tee(other)[0]
             else:
                 yield other
@@ -131,19 +133,21 @@ class ArkoWrapper(Generic[T]):
 
     def __eq__(self, other: Any) -> bool:
         """定义操作符(==)的行为。"""
-        if any([
-            isinstance(other, ArkoWrapper) and self.__root__ == other.__root__,
-            isinstance(other, Iterable) and self.__root__ == other
-        ]):
+        if isinstance(other, Sized) and self.length != len(other):
+            return False
+        if isinstance(other, Hashable) and self.__hash__() == hash(other):
             return True
-        if isinstance(other, (ArkoWrapper, Iterable)):
-            for i in self.zip(other):
+        if isinstance(other, Iterable):
+            for i in self.zip_longest(ArkoWrapper(other)):
                 if i[0] != i[1]:
                     return False
             return True
         return self == [other]
 
-    def __copy__(self: M) -> "M":
+    def __copy__(self: Wrapper) -> "Wrapper":
+        return self.__class__(self.__root__)
+
+    def __deepcopy__(self: Wrapper, *args) -> "Wrapper":
         """定义对类的实例使用 copy.copy() 时的行为"""
         return self.__class__(self._tee())
 
@@ -155,10 +159,10 @@ class ArkoWrapper(Generic[T]):
                 return self.__root__[index]
             try:
                 return self.slice(
-                    *filter(None, [index.start, index.step, index.stop])
+                    *filter(None, [index.start, index.stop, index.step])
                 )
             except ValueError:
-                return list(self._max_gen()).__getitem__(index)
+                return self.__class__(list(self._max_gen()).__getitem__(index))
         try:
             if (index := int(index)) > 0:
                 target = self._tee()
@@ -168,73 +172,93 @@ class ArkoWrapper(Generic[T]):
             iter_values = iter(target)
             time = -1
             while value := next(iter_values):
-                try:
-                    if (time := time + 1) == index:
-                        return value
-                except StopIteration:
-                    raise ValueError(f"Out of range: {index}")
+                if (time := time + 1) == index:
+                    return value
+        except StopIteration:
+            raise ValueError(f"Out of range: {index}")
         except Exception:
             raise IndexError("Unsupported indexing for iterable")
 
+    def __hash__(self) -> int:
+        return hash(self._tee())
+
     def __index__(self) -> int:
         """实现当对象用于切片表达式时到一个整数的类型转换。"""
         return self.__len__()
 
-    def __iter__(self) -> T:
+    def __iter__(self) -> Iterator[T]:
         """返回当前的迭代器。"""
-        return self._tee().__iter__()
+        yield from self._tee()
+
+    _len_cache: ClassVar[dict[int, int]] = {}
 
     def __len__(self) -> int:
         """返回当前的迭代器的长度，如果无限的话，则回返回最大操作次数。"""
+        if (
+                isinstance(self.__root__, Hashable)
+                and
+                (
+                        (hash_value := hash(self.__root__))
+                        in
+                        self.__class__._len_cache
+                )
+        ):
+            return self.__class__._len_cache[hash_value]
         if isinstance(self.__root__, Sized):
-            return len(list(self._tee()))
+            length = len(list(self._tee()))
         else:
             length = 0
             for _ in self._tee():
                 length += 1
                 if length >= self._max:
                     return self._max
-            return length
+        if isinstance(self.__root__, Hashable):
+            self.__class__._len_cache[hash(self.__root__)] = length
+        return length
 
     def __matmul__(self, other: Any) -> T:
         """定义操作符(@)的行为。"""
         return self.__getitem__(other)
 
-    def __mul__(self: M, times: Union[int, float, str]) -> "M":
+    def __mul__(self: Wrapper, times: Union[int, float, str]) -> "Wrapper":
         """实现乘法操作"""
+        if (
+                isinstance(times, SupportsIndex)
+                and
+                (times := int(float(times))) <= 0
+        ):
+            raise ValueError(f"'times' cannot be negative: {times}")
         try:
-            if (times := int(float(times))) <= 0:
-                raise ValueError(f"'times' cannot be negative: {times}")
             return self.__class__(
-                chain.from_iterable(repeat(tuple(self._tee()), times))
+                chain.from_iterable(repeat(tuple(self._tee()), int(times)))
             )
-        except (ValueError, TypeError):
+        except Exception:
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def __neg__(self: M) -> "M":
+    def __neg__(self: Wrapper) -> "Wrapper":
         """定义取负操作"""
         return self.__reversed__()
 
-    def __reversed__(self: M) -> "M":
+    def __reversed__(self: Wrapper) -> "Wrapper":
         """定义反转"""
         if isinstance(self.__root__, Reversible):
             from copy import deepcopy as copy
             return self.__class__(reversed(copy(self.__root__)))
         else:
-            try:
-                return self.__class__(reversed(list(self._max_gen())))
-            except (ValueError, TypeError):
-                raise TypeError(
-                    f"The iter '{self.__root__}' is not 'Reversible.'")
+            return self.__class__(reversed(list(self._max_gen())))
 
-    def __rshift__(self: M, target: Any) -> C:
+    def __rshift__(
+            self: Wrapper,
+            target: Union[Callable[[Iterable[T], ...], C], Sequence]
+    ) -> C:
         """实现右移位运算符 >>"""
         if isinstance(target, type) or callable(target):
             result: C = self.collect(target)
         elif isinstance(target, Sequence):
-            result: C = self.collect(type(target))
+            # noinspection PyArgumentList,PyProtectedMember
+            result: C = target.__class__(self.__radd__(target)._tee())
         else:
-            raise ValueError(f"Unsupported value or type: '{target}'")
+            raise TypeError(f"Unsupported value or type: '{target}'")
         return result
 
     @property
@@ -251,15 +275,16 @@ class ArkoWrapper(Generic[T]):
         return self._max
 
     @max_operate_time.setter
-    def max_operate_time(self, max_operate_time: int):
+    def max_operate_time(self, max_operate_time: int) -> NoReturn:
         if max_operate_time <= 0:
             raise ValueError(f"Requires a positive number: {max_operate_time}")
         self._max = max_operate_time
 
     def accumulate(
-            self: M, func: Optional[Callable[[T, T], C]] = operator.add, *,
+            self: Wrapper, func: Optional[Callable[[T, T], C]] = operator.add,
+            *,
             initial: Optional[int] = None
-    ) -> "M[C]":
+    ) -> Wrapper:
         return self.__class__(accumulate(self._tee(), func, initial=initial))
 
     def all(self) -> bool:
@@ -282,7 +307,7 @@ class ArkoWrapper(Generic[T]):
         except StopIteration:
             return False
 
-    def chain(self: M, *iterables: Iterable[E]) -> "M[Union[T, E]]":
+    def chain(self: Wrapper, *iterables: Iterable[E]) -> Wrapper:
         """创建一个迭代器，它首先返回第一个可迭代对象中所有元素，接着返回下一个可迭代对象中所有元素，直到耗尽所有可迭代对象中的元素。"""
         return self.__class__(chain(self._tee(), *iterables))
 
@@ -294,23 +319,28 @@ class ArkoWrapper(Generic[T]):
         """以整个迭代器作为参数，于给定的 func 函数中进行运算"""
         return func(self._tee(), *args, **kwargs)
 
-    def combinations(self: M, r: int = 2) -> "M[combinations[tuple[T, T]]]":
+    def combinations(self: Wrapper, r: int = 2) -> Wrapper:
+        """返回由元素组成长度为 r 的子序列"""
         return self.__class__(combinations(self._tee(), r))
 
-    def combinations_with_replacement(self: M, r: int = 2) -> "M":
+    def combinations_with_replacement(
+            self: Wrapper, r: int = 2
+    ) -> Wrapper:
+        """返回由元素组成的长度为 r 的子序列，允许每个元素可重复出现。"""
         return self.__class__(combinations_with_replacement(self._tee(), r))
 
-    def compress(self: M, selectors: Iterable) -> "M":
+    def compress(self: Wrapper, selectors: Iterable) -> Wrapper:
+        """返回元素中经 selectors 真值测试为 True 的元素"""
         return self.__class__(compress(self._tee(), selectors))
 
-    def cycle(self: M) -> "M":
+    def cycle(self: Wrapper) -> "Wrapper":
         return self.__class__(cycle(self._tee()))
 
-    def drop_while(self: M, func: Callable) -> "M":
+    def drop_while(self: Wrapper, func: Callable) -> "Wrapper":
         return self.__class__(dropwhile(func, self._tee()))
 
-    def enumerate(self: M) -> "M[tuple[int, T]]":
-        def generator() -> Generator[tuple[int, T]]:
+    def enumerate(self: Wrapper) -> Wrapper:
+        def generator() -> Iterator[tuple[int, T]]:
             iter_values = iter(self._tee())
             index = 0
             for _ in range(self.max_operate_time):
@@ -323,32 +353,61 @@ class ArkoWrapper(Generic[T]):
 
         return self.__class__(generator())
 
-    def filter(self: M, func: Callable[[T], Any]) -> "M[T]":
+    def fill(
+            self: Wrapper,
+            num: int,
+            factory: Union[C, Callable[[], C]] = None
+    ) -> Wrapper:
+        if num > self._max:
+            raise ValueError("'num' cannot exceed the maximum number of '_max'")
+        elif num < 0:
+            raise ValueError("'num' must be a positive number.")
+
+        def generator() -> Iterator[C]:
+            yield from self._tee()  # typed: T
+            for _ in range(num):
+                yield factory() if callable(factory) else factory  # typed: C
+
+        return self.__class__(generator())
+
+    def fill_to(
+            self: Wrapper,
+            num: int,
+            factory: Union[C, Callable[[], C]] = None
+    ) -> Wrapper:
+        if num < self.length:
+            raise ValueError("'num' cannot be less than its own length.")
+        if num == (length := self.length):
+            return self
+        else:
+            return self.fill(num - length, factory=factory)
+
+    def filter(self: Wrapper, func: Callable[[T], Any]) -> Wrapper:
         return self.__class__(filter(func, self._tee()))
 
-    def filter_false(self: M, func: Callable[[T], Any]) -> "M[T]":
+    def filter_false(self: Wrapper, func: Callable[[T], Any]) -> Wrapper:
         return self.__class__(filterfalse(func, self._tee()))
 
     def find(
             self, target: Any, *, full: Optional[bool] = False
-    ) -> Generator[int]:
+    ) -> Iterator[int]:
         for t in self.enumerate():
             if t[1] == target:
                 yield t[0]
                 if not full:
                     break
 
-    def flat(self: M, depth: int = -1) -> "M":
-        def generate(iterator: Iterable, times: int) -> Generator:
+    def flat(self: Wrapper, depth: int = -1) -> Wrapper:
+        def generator(iterator: Iterable, times: int) -> Iterator[T]:
             for i in iterator:
                 if isinstance(i, Iterable) and not isinstance(i, str) and times:
-                    yield from generate(i, times - 1)
+                    yield from generator(i, times - 1)
                 else:
                     yield i
 
-        return self.__class__(generate(self._tee(), depth))
+        return self.__class__(generator(self._tee(), depth))
 
-    def group(self: M, n: int, fill_value: Any = None) -> "M":
+    def group(self: Wrapper, n: int, fill_value: Any = None) -> "Wrapper":
         iter_values = [iter(self._tee())] * n
         return self.__class__(zip_longest(*iter_values, fillvalue=fill_value))
 
@@ -363,13 +422,13 @@ class ArkoWrapper(Generic[T]):
             ...
         return result
 
-    def map(self: M, func: Callable) -> "M":
+    def map(self: Wrapper, func: Callable) -> "Wrapper":
         return self.__class__(map(func, self._tee()))
 
     def mutate(
-            self: M, func: Callable[[Iterable[T], ...], Iterable] = list,
+            self: Wrapper, func: Callable[[Iterable[T], ...], Iterable] = list,
             *args, **kwargs
-    ) -> "M":
+    ) -> "Wrapper":
         return self.__class__(func(self._tee(), *args, **kwargs))
 
     def print(
@@ -396,26 +455,24 @@ class ArkoWrapper(Generic[T]):
             print_func(len(end) * '\b')
         return self
 
-    def range(self: M, start: Optional[int] = 0) -> "M":
-        def generator() -> Generator[int]:
+    def range(self: Wrapper, start: Optional[int] = 0) -> Wrapper:
+        def generator() -> Iterator[T]:
             iter_values = iter(self._tee())
-            index = start
             try:
-                while index > 0:
+                for _ in range(start):
                     next(iter_values)
-                    index -= 1
                 index = start
                 for _ in range(self.max_operate_time):
                     next(iter_values)
                     yield index
                     index += 1
             except StopIteration:
-                return
+                ...
 
         return self.__class__(generator())
 
     def remove(self, target: Any) -> "ArkoWrapper":
-        def generator():
+        def generator() -> Iterator[T]:
             iter_values = iter(self._tee())
             is_sequence = isinstance(target, Sequence)
             removed = False
@@ -436,8 +493,8 @@ class ArkoWrapper(Generic[T]):
         return ArkoWrapper(generator())
 
     def repeat(
-            self: M, times: Optional[Union[int, float, str]] = None
-    ) -> "M[T]":
+            self: Wrapper, times: Optional[Union[int, float, str]] = None
+    ) -> Wrapper:
         if times is None:
             return self.__class__(
                 chain.from_iterable(
@@ -456,25 +513,25 @@ class ArkoWrapper(Generic[T]):
         else:
             raise TypeError(f"Unsupported Type: {type(times)}.")
 
-    def reverse(self: M) -> "M":
+    def reverse(self: Wrapper) -> "Wrapper":
         return self.__reversed__()
 
     @overload
-    def slice(self: M, stop: int) -> "M":
+    def slice(self: Wrapper, stop: int) -> "Wrapper":  # pragma: no cover
         ...
 
     @overload
     def slice(
-            self: M, start: int, stop: int, step: Optional[int] = 1
-    ) -> "M":
+            self: Wrapper, start: int, stop: int, step: Optional[int] = 1
+    ) -> "Wrapper":  # pragma: no cover
         ...
 
-    def slice(self: M, *args) -> "M":
-        return self.__class__(islice(self._tee(), *args))
+    def slice(self: Wrapper, *args, **kwargs) -> "Wrapper":
+        return self.__class__(islice(self._tee(), *args, **kwargs))
 
-    def search(self, sub: Sized) -> Generator[int]:
+    def search(self, sub: Sized) -> Generator:
         target = self.tee()
-        sub = ArkoWrapper(sub)
+        sub: ArkoWrapper = ArkoWrapper(sub)
         partial: list[int] = [0]
         for i in sub.range(1):
             j = partial[i - 1]
@@ -494,22 +551,32 @@ class ArkoWrapper(Generic[T]):
                 j = partial[j - 1]
 
     def sort(
-            self: M, key: Optional[Callable] = None, reverse: bool = False
-    ) -> "M":
+            self: Wrapper, key: Optional[Callable] = None, reverse: bool = False
+    ) -> "Wrapper":
         return self.__class__(sorted(self._tee(), key=key, reverse=reverse))
 
-    def starmap(self: M, func: Callable) -> "M":
+    def starmap(self: Wrapper, func: Callable) -> "Wrapper":
         return self.__class__(starmap(func, self._tee()))
 
+    def take_while(
+            self, func: Union[Any, Callable[[T], bool]] = True
+    ) -> Wrapper:
+        if callable(func):
+            return self.__class__(takewhile(func, self._tee()))
+        elif bool(func):
+            return self.__deepcopy__()
+        else:
+            return self.__class__()
+
     def tee(
-            self: M, n: Optional[int] = None
-    ) -> Union["M", Generator["M"]]:
+            self: Wrapper, n: Optional[int] = None
+    ) -> Union["Wrapper", Iterable["Wrapper"]]:
         if n is None:
             return self.__class__(self._tee())
         return (self.__class__(item) for item in tee(self.__root__, n))
 
-    def unique(self: M) -> "M":
-        def generator() -> Generator:
+    def unique(self: Wrapper) -> "Wrapper":
+        def generator() -> Iterator[T]:
             for k, g in groupby(self._tee()):
                 yield k
 
@@ -519,8 +586,19 @@ class ArkoWrapper(Generic[T]):
         return self._tee()
 
     def zip(
-            self: M, *iterables: Iterable[E], strict: Optional[bool] = False
-    ) -> "M[Union[T, E]]":
-        if sys.version_info >= (3, 10):
+            self: Wrapper, *iterables: Iterable[E],
+            strict: Optional[bool] = False
+    ) -> Wrapper:
+        if sys.version_info >= (3, 10):  # pragma: no cover
             return self.__class__(zip(self._tee(), *iterables, strict=strict))
         return self.__class__(zip(self._tee(), *iterables))
+
+    def zip_longest(
+            self: Wrapper, *iterables: Iterable[E],
+            fill_value: Optional[C] = None
+    ) -> Wrapper:
+        return self.__class__(
+            zip_longest(
+                self.tee(), *ArkoWrapper(iterables).tee(), fillvalue=fill_value
+            )
+        )
